import QtQuick 2.3
import "Theme.js" as Theme
import "functions.js" as F

Rectangle {
    color: Theme.background_color;

    visible: false;
    property var clickedModel;
    property var navModel;

    signal showNav();
    signal selectedAirspace(string name);

    Text {
        id: pageHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: Theme.margin
        horizontalAlignment: Text.AlignRight;
        //% "Airspaces"
        text: "Vzdušné prostory"
        color: Theme.header_color
        font.pixelSize: Theme.header_pixelSize
    }


    ListView {
        anchors.top: pageHeader.bottom
        anchors.left: parent.left;
        anchors.right: parent.right;
        anchors.bottom: parent.bottom;
        anchors.topMargin: Theme.margin
        anchors.bottomMargin: Theme.margin
        clip: true;
        spacing: Theme.marginSmall

        model: clickedModel
        delegate: DialogButton {
            anchors.left: parent.left;
            anchors.right: parent.right;
            icon: "./data/"+ model.type + ".png";
            title: model.name;
            color: (model.events > 0) ? "#a40000": Theme.background_color;
            subtitle: formatSubtitle(clickedModel.get(index));

            onClicked: {

                if (model.type !== "POLY") {
                    navModel.append({
                                        "name": model.name,
                                        "type": model.type,
                                        "lat": model.lat,
                                        "lon": model.lon,
                                    })
                    showNav();
                } else {
                    selectedAirspace(model.designator);

                }
            }

        }
    }

    function formatSubtitle(item) {
        if (item === undefined) {
            return "";
        }
        switch(item.type) {
        case "POLY":
            return item.subtype + "; " + item.low + "; " + item.up +  ((item.events > 0) ? " - ACTIVE" : "") ;
        case "AD":
            return item.ele + " " + item.eleu + " - " + item.directions + ((item.events > 0) ? " (NOTAMs: " + item.events +")" : "");
        default:
            return F.formatCoordinate(item.lat, item.lon, {coordinateFormat: "DMS"});
        }
    }

}
