import QtQuick 2.3
import "Theme.js" as Theme
import "functions.js" as F

Rectangle {
    color: Theme.background_color;

    property real currentLatitude;
    property real currentLongitude;
    property bool currentPositionValid: false

    property var navigationModel;


    function computePrevDistance(index, lat, lon) {
        if (index <= 0) {
            return 0;
        }

        var prev = navigationModel.get(index-1);
        return F.getDistanceTo(lat, lon, prev.lat, prev.lon)

    }

    function computePrevBearing(index, lat, lon) {
        if (index <= 0) {
            return F.getBearingTo(currentLatitude, currentLongitude, lat, lon)
        }

        var prev = navigationModel.get(index-1);
        return F.getBearingTo(prev.lat, prev.lon, lat, lon)
    }


    function computePathDistance(index, lat, lon) {
        var aLat = lat;
        var aLon = lon;
        var distance = 0;
        for (var i = index-1; i >= 0; i--) {
            var b = navigationModel.get(i);
            distance += F.getDistanceTo(aLat, aLon, b.lat, b.lon);
            aLat = b.lat;
            aLon = b.lon;
        }
        distance += F.getDistanceTo(aLat, aLon, currentLatitude, currentLongitude);
        return distance;
    }

    Text {
        id: pageHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: Theme.margin
        horizontalAlignment: Text.AlignRight;
        //% "Navigation"
        text: "Navigace"
        color: Theme.header_color
        font.pixelSize: Theme.header_pixelSize
    }



    Flickable {
        anchors.top: pageHeader.bottom
        anchors.left: parent.left;
        anchors.right: parent.right;
        anchors.bottom: parent.bottom;
        anchors.topMargin: Theme.margin
        anchors.bottomMargin: Theme.margin
        contentHeight: column.height
        clip: true;

        Column {
            id: column
            anchors.left: parent.left;
            anchors.right: parent.right;
            spacing: Theme.marginSmall

            DialogButton {
                anchors.left: parent.left;
                anchors.right: parent.right;
                icon: "./data/POI.png"
                title: "Současná poloha"
                subtitle: currentPositionValid
                          ? F.formatCoordinate(currentLatitude, currentLongitude, {coordinateFormat: "DMS"})
                          : "neznámá"
            }

            Repeater {
                model: navigationModel
                delegate: DialogButton {
                    property real distance: currentPositionValid ? F.getDistanceTo(model.lat, model.lon, currentLatitude, currentLongitude) : 0;
                    property real pathDistance: currentPositionValid ? computePathDistance(index, model.lat, model.lon) : 0;
                    property real prevBearing: computePrevBearing(index, model.lat, model.lon)

                    anchors.left: column.left;
                    anchors.right: column.right;
                    title: model.name
                    icon: "./data/"+ model.type + ".png";

                    subtitle: (currentPositionValid ?
                                   (
                                       F.formatDistance(pathDistance, {distanceUnit: "m"} )
                                       + " - "
                                       + Math.round(prevBearing) + "°"
                                       + " - "

                                       ) : "")
                              + F.formatCoordinate(model.lat, model.lon, {coordinateFormat: "DMS"})
                    onClicked: {
                        navigationModel.remove(index, 1);
                    }
                }
            }
        }

    }


}
