import QtQuick 2.2
import cz.mlich 1.0
import "functions.js" as F

Rectangle {
    id: pinchmap;

    property real zoomLevel: 7;
    property int zoomLevelInt: Math.floor(zoomLevel);
    property real zoomLevelReminder: zoomLevel - zoomLevelInt
    property int oldZoomLevel: 99
    property int maxZoomLevel: 17;
    property int minZoomLevel: 2;
    property real tileScaleFactor: 2
    property int tileSize: (128 + (128 * zoomLevelReminder)) * tileScaleFactor;
    //        property int tileSize: 256 * tileScaleFactor;

    property int cornerTileX: 32;
    property int cornerTileY: 32;
    property int widthHeight: Math.max(width, height)*3/2
    property int numTilesX: rotationEnabled ? (Math.ceil(widthHeight/tileSize) + 1) : (Math.ceil(width/tileSize) + 1);
    property int numTilesY: rotationEnabled ? (Math.ceil(widthHeight/tileSize) + 1) : (Math.ceil(height/tileSize) + 1);
    property int maxTileNo: Math.pow(2, zoomLevelInt) - 1;

    property bool rotationEnabled: false;

    property bool pageActive: true;

    property double latitude: 49.803575
    property double longitude: 15.475555
    property variant scaleBarLength: getScaleBarLength(latitude);
    property real angle: 0;

    property string url: MAP_LOCAL_STORAGE;
    //    property string url: "http://a.tile.openstreetmap.org/%(zoom)d/%(x)d/%(y)d.png";


    property real pointRemoveDistance: 3000;

    property int earthRadius: 6371000

    //property bool pageActive: (status == PageStatus.active);
    property bool needsUpdate: false
    signal pannedManually

    property alias airportModelA: airportModel;
    property alias navigationModelA: navigationModel;
    property alias clickedModelA: clickedModel;
    property alias warningModelA: warningModel;
    property variant air;

    property alias positionIndicator: planePosition
    property real timeForFuturePos: 300; // 5 minutes
    property string selectedAirspace;

    signal clickedIntoMap(real x, real y);

    transform: Rotation {
        id: rot
        origin.x: pinchmap.width/2
        origin.y: pinchmap.rotationEnabled ? 0.75*pinchmap.height :pinchmap.height/2
        angle: pinchmap.rotationEnabled ? pinchmap.angle : 0;
    }

    onAirChanged: {
        if (air === undefined) {
            return;
        }

        if (air.airportHeliport === undefined) {
            return;
        }

        var airportHeliport = air.airportHeliport;
        airportModel.clear();
        for (var i = 0; i < airportHeliport.length; i++) {
            airportModel.append(airportHeliport[i])
        }


        requestUpdateDetails();
    }

    onTileSizeChanged: {
        canvasRequestUpdate()
    }

    onCornerTileXChanged: {
        canvasRequestUpdate();
    }
    onCornerTileYChanged: {
        canvasRequestUpdate();

    }

    onMaxZoomLevelChanged: {
        if (pinchmap.maxZoomLevel < pinchmap.zoomLevel) {
            setZoomLevel(maxZoomLevel);
        }
    }

    onPageActiveChanged:  {
        if (pageActive && needsUpdate) {
            needsUpdate = false;
            pinchmap.setCenterLatLon(pinchmap.latitude, pinchmap.longitude);
            canvasRequestUpdate();
        }
    }
    
    onWidthChanged: {
        if (!pageActive) {
            needsUpdate = true;
        } else {
            pinchmap.setCenterLatLon(pinchmap.latitude, pinchmap.longitude);
            canvasRequestUpdate();
        }
    }

    onHeightChanged: {
        if (!pageActive) {
            needsUpdate = true;
        } else {
            pinchmap.setCenterLatLon(pinchmap.latitude, pinchmap.longitude);
            canvasRequestUpdate();
        }
    }



    function setZoomLevel(z) {
        setZoomLevelPoint(z, pinchmap.width/2, pinchmap.height/2);
    }

    function zoomIn() {
        setZoomLevel(pinchmap.zoomLevel + 1)
    }

    function zoomOut() {
        setZoomLevel(pinchmap.zoomLevel - 1)
    }

    function setZoomLevelPoint(z, x, y) {
        if (z === zoomLevel) {
            return;
        }
        if (z < pinchmap.minZoomLevel || z > pinchmap.maxZoomLevel) {
            return;
        }
        var p = getCoordFromScreenpoint(x, y);
        zoomLevel = z;
        setCoord(p, x, y);
    }

    function pan(dx, dy) {
        map.offsetX -= dx;
        map.offsetY -= dy;
    }

    function panEnd() {
        var changed = false;
        var threshold = pinchmap.tileSize;

        while (map.offsetX < -threshold) {
            map.offsetX += threshold;
            cornerTileX += 1;
            changed = true;
        }
        while (map.offsetX > threshold) {
            map.offsetX -= threshold;
            cornerTileX -= 1;
            changed = true;
        }

        while (map.offsetY < -threshold) {
            map.offsetY += threshold;
            cornerTileY += 1;
            changed = true;
        }
        while (map.offsetY > threshold) {
            map.offsetY -= threshold;
            cornerTileY -= 1;
            changed = true;
        }
        updateCenter();
    }

    function updateCenter() {
        var l = getCenter()
        longitude = l[1]
        latitude = l[0]
    }

    function requestUpdate() {
        var start = getCoordFromScreenpoint(0,0)
        var end = getCoordFromScreenpoint(pinchmap.width,pinchmap.height)
        //         controller.updateGeocaches(start[0], start[1], end[0], end[1])
        //        console.debug("Update requested.")
        canvasRequestUpdate();
    }

    function requestUpdateDetails() {
        var start = getCoordFromScreenpoint(0,0)
        var end = getCoordFromScreenpoint(pinchmap.width,pinchmap.height)
        //        controller.downloadGeocaches(start[0], start[1], end[0], end[1])
        console.debug("Download requested.")
    }

    function getScaleBarLength(lat) {
        var destlength = width/5;
        var mpp = getMetersPerPixel(lat);
        var guess = mpp * destlength;
        var base = 10 * -Math.floor(Math.log(guess)/Math.log(10) + 0.00001)
        var length_meters = Math.round(guess/base)*base
        var length_pixels = length_meters / mpp
        return [length_pixels, length_meters]
    }

    function getMetersPerPixel(lat) {
        return Math.cos(lat * Math.PI / 180.0) * 2.0 * Math.PI * earthRadius / (256 * (maxTileNo + 1))
    }

    function deg2rad(deg) {
        return deg * (Math.PI /180.0);
    }

    function deg2num(lat, lon) {
        var rad = deg2rad(lat % 90.0);
        var n = maxTileNo + 1;
        var xtile = ((lon % 180.0) + 180.0) / 360.0 * n;
        var ytile = (1.0 - Math.log(Math.tan(rad) + (1.0 / Math.cos(rad))) / Math.PI) / 2.0 * n;
        return [xtile, ytile];
    }

    function setLatLon(lat, lon, x, y) {
        var oldCornerTileX = cornerTileX
        var oldCornerTileY = cornerTileY
        var tile = deg2num(lat, lon);
        var cornerTileFloatX = tile[0] + (map.rootX - x) / tileSize // - numTilesX/2.0;
        var cornerTileFloatY = tile[1] + (map.rootY - y) / tileSize // - numTilesY/2.0;
        cornerTileX = Math.floor(cornerTileFloatX);
        cornerTileY = Math.floor(cornerTileFloatY);
        map.offsetX = -(cornerTileFloatX - Math.floor(cornerTileFloatX)) * tileSize;
        map.offsetY = -(cornerTileFloatY - Math.floor(cornerTileFloatY)) * tileSize;
        updateCenter();
    }

    function setCoord(c, x, y) {
        setLatLon(c[0], c[1], x, y);
    }

    function setNavLatLon(lat, lon) { // rot.origin instead of center
        setLatLon(lat, lon, rot.origin.x, rot.origin.y);
    }

    function setCenterLatLon(lat, lon) {
        setLatLon(lat, lon, pinchmap.width/2, pinchmap.height/2);
    }

    function setCenterCoord(c) {
        setCenterLatLon(c[0], c[1]);
    }

    function getCoordFromScreenpoint(x, y) {
        var realX = - map.rootX - map.offsetX + x;
        var realY = - map.rootY - map.offsetY + y;
        var realTileX = cornerTileX + realX / tileSize;
        var realTileY = cornerTileY + realY / tileSize;
        return num2deg(realTileX, realTileY);
    }

    function getScreenpointFromCoord(lat, lon) {
        var tile = deg2num(lat, lon)
        var realX = (tile[0] - cornerTileX) * tileSize
        var realY = (tile[1] - cornerTileY) * tileSize
        var x = realX + map.rootX + map.offsetX
        var y = realY + map.rootY + map.offsetY
        return [x, y]
    }

    function getMappointFromCoord(lat, lon) {
        var tile = deg2num(lat, lon)
        var realX = (tile[0] - cornerTileX) * tileSize
        var realY = (tile[1] - cornerTileY) * tileSize
        return [realX, realY]
        
    }

    function getCenter() {
        return getCoordFromScreenpoint(pinchmap.width/2, pinchmap.height/2);
    }

    function sinh(aValue) {
        return (Math.pow(Math.E, aValue)-Math.pow(Math.E, -aValue))/2;
    }

    function num2deg(xtile, ytile) {
        var n = Math.pow(2, zoomLevelInt);
        var lon_deg = xtile / n * 360.0 - 180;
        var lat_rad = Math.atan(sinh(Math.PI * (1 - 2 * ytile / n)));
        var lat_deg = lat_rad * 180.0 / Math.PI;
        return [lat_deg % 90.0, lon_deg % 180.0];
    }

    function tileUrl(tx, ty) {
        if ((url === undefined) || (url === "")) {
            return "./data/noimage-disabled.png"
        }

        if (tx < 0 || tx > maxTileNo) {
            return "./data/noimage.png"
        }

        if (ty < 0 || ty > maxTileNo) {
            return "./data/noimage.png"
        }


        var res = Qt.resolvedUrl(F.getMapTile(url, tx, ty, zoomLevelInt));

        if (filereader.is_local_file(res) && !filereader.file_exists(res)) { // do not open non existing image
            //            return "./data/noimage.png"
            return "";
        }

        return res;

    }

    function imageStatusToString(status) {
        switch (status) {
        case Image.Ready: return qsTr("Ready");
        case Image.Null: return qsTr("Not Set");
        case Image.Error: return qsTr("Error");
        case Image.Loading: return qsTr("Loading...");
        default: return qsTr("Unknown");
        }
    }

    ListModel {
        id: airportModel;
    }

    ListModel {
        id: navigationModel;
        onCountChanged: {
            trajectoryCanvasRequestUpdate();
        }
    }

    ListModel {
        id: clickedModel;
    }

    ListModel {
        id: warningModel;
    }



    Grid {

        id: map;
        columns: numTilesX;
        width: numTilesX * tileSize;
        height: numTilesY * tileSize;
        property int rootX: -(width - parent.width)/2;
        property int rootY: -(height - parent.height)/2;
        property int offsetX: 0;
        property int offsetY: 0;
        x: rootX + offsetX;
        y: rootY + offsetY;




        Repeater {
            id: tiles
            model: (pinchmap.numTilesX * pinchmap.numTilesY);
            Rectangle {
                id: tile
                property alias source: img.source;
                property int tileX: cornerTileX + (index % numTilesX)
                property int tileY: cornerTileY + Math.floor(index / numTilesX)
                Rectangle {
                    id: progressBar;
                    property real p: 0;
                    height: 16;
                    width: parent.width - 32;
                    anchors.centerIn: img;
                    color: "#c0c0c0";
                    border.width: 1;
                    border.color: "#000000";
                    Rectangle {
                        anchors.left: parent.left;
                        anchors.margins: 2;
                        anchors.top: parent.top;
                        anchors.bottom: parent.bottom;
                        width: (parent.width - 4) * progressBar.p;
                        color: "#000000";
                    }
                    visible: (img.status !== Image.Ready) && (img.status !== Image.Null)
                }
                Text {
                    anchors.left: parent.left
                    anchors.leftMargin: 16
                    y: parent.height/2 - 32
                    text: imageStatusToString(img.status)
                    visible: (img.status !== Image.Ready) && (img.status !== Image.Null)
                }
                //                Image {
                //                    anchors.fill: parent;
                //                    visible: (img.status === Image.Null)
                //                    source: "./data/noimage.png"
                //                }


                Image {
                    id: img;
                    anchors.fill: parent;
                    onProgressChanged: { progressBar.p = progress }
                    source: tileUrl(tileX, tileY)
                }


                width: tileSize;
                height: tileSize;
                color: "#c0c0c0" ;
            }


        }

    }

    Canvas {
        id: canvas
        anchors.fill: map; // map == Grid

        renderStrategy: Canvas.Cooperative;

        function drawPolyline(ctx, polygon) {

            var point = polygon[0];
            var screenPoint = getMappointFromCoord(point.lat, point.lon)

            ctx.beginPath();
            ctx.moveTo(screenPoint[0], screenPoint [1]);

            for (var i = 1; i < polygon.length; i++) {
                point = polygon[i];
                screenPoint = getMappointFromCoord(point.lat, point.lon)
                ctx.lineTo(screenPoint[0], screenPoint[1])
            }
            ctx.closePath();

        }

        function drawCTR(ctx, polygon) {
            drawPolyline(ctx, polygon);
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ff000080";
            ctx.stroke();

        }

        function drawTMA(ctx, polygon) {
            //                        drawPolyline(ctx, polygon)
            //                        ctx.clip();

            //                        ctx.lineWidth = 20;
            //                        ctx.strokeStyle = "#33000080";

            //                        drawPolyline(ctx, polygon);
            //                        ctx.stroke();

            //                        ctx.lineWidth = 3;
            //                        ctx.strokeStyle = "#ff000080";
            //                        drawPolyline(ctx, polygon);
            //                        ctx.stroke();


            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ff000080";
            ctx.stroke();
            ctx.fillStyle = "#33000080";
            ctx.fill();

        }

        function drawNOTAM(ctx, polygon) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffff0000";
            ctx.stroke();
            ctx.fillStyle = "#33ff0000";
            ctx.fill();
        }

        function drawTRA(ctx, polygon, isActive) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffff0000";
            ctx.stroke();
            if (isActive) {
                ctx.fillStyle = "#33ff0000";
                ctx.fill();
            }
        }

        function drawTSA(ctx, polygon, isActive) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffc4a000";
            ctx.stroke();
            if (isActive) {
                ctx.fillStyle = "#66c4a000";
                ctx.fill();
            }
        }

        function drawR(ctx, polygon, isActive) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffff0000";
            ctx.stroke();
            if (isActive) {
                ctx.fillStyle = "#33ff0000";
                ctx.fill();
            }
        }

        function drawD(ctx, polygon) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffff0000";
            ctx.stroke();
            ctx.fillStyle = "#33ff0000";
            ctx.fill();
        }

        function drawP(ctx, polygon) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ffff0000";
            ctx.stroke();
            ctx.fillStyle = "#33ff0000";
            ctx.fill();
        }

        function drawFIR(ctx, polygon) {
            drawPolyline(ctx, polygon)
            //            ctx.lineWidth = 1;
            ctx.strokeStyle = "#ff000000";
            ctx.stroke();

        }

        onPaint: {
            //            console.count("canvasCount")
            console.time("canvas-onPaint")

            var ctx = canvas.getContext("2d");

            ctx.clearRect(0, 0, canvas.width, canvas.height);

            // 4 rohy kvuli rotaci
            var neCorner = getCoordFromScreenpoint(0, 0);
            var nwCorner = getCoordFromScreenpoint(width, 0);
            var swCorner = getCoordFromScreenpoint(0, height);
            var seCorner = getCoordFromScreenpoint(width, height);
            var cornerLatMin = Math.min(neCorner[0], nwCorner[0], swCorner[0], seCorner[0]);
            var cornerLonMin = Math.min(neCorner[1], nwCorner[1], swCorner[1], seCorner[1]);
            var cornerLatMax = Math.max(neCorner[0], nwCorner[0], swCorner[0], seCorner[0]);
            var cornerLonMax = Math.max(neCorner[1], nwCorner[1], swCorner[1], seCorner[1]);

            if ((air === undefined) || air.airspace === undefined) {
                return;
            }

            var airspaces = air.airspace;
            for (var i = 0; i < airspaces.length; i++) {
                ctx.save();
                var airspace = airspaces[i];

                ctx.lineWidth = (airspace.designator === selectedAirspace) ? 3 : 1;

                var geometry = airspace.geometry;

                if (
                        (geometry.max_lat < cornerLatMin) ||
                        (geometry.max_lon < cornerLonMin) ||
                        (geometry.min_lat > cornerLatMax) ||
                        (geometry.min_lon > cornerLonMax)
                        ) {
                    continue;
                }

                var points = geometry.points;
                var atype = airspace.type
                var statusActive = (airspace.tempdelta.length > 0);
                switch (atype) {
                case "FIR":
                    drawFIR(ctx, points);
                    break;
                case "TMA":
                    drawTMA(ctx, points);
                    break;
                case "CTR":
                    drawCTR(ctx, points);
                    break;
                case "TRA":
                    drawTRA(ctx, points, statusActive);
                    break;
                case "TSA":
                    drawTSA(ctx, points, statusActive);
                    break;
                case "R":
                    drawR(ctx, points, statusActive);
                    break;
                case "D":
                    drawD(ctx, points);
                    break;
                case "P":
                    drawP(ctx, points);
                    break;
                default:
                    console.log("unknown polygon type " + atype)
                    drawNOTAM(ctx, points);
                    break;
                }
                ctx.restore();
            }


            if (navigationModel.count > 0) {
                ctx.beginPath();
                // paint current trajectory
                var screenPoint = getMappointFromCoord(planePosition.latitude, planePosition.longitude)
                ctx.moveTo(screenPoint[0], screenPoint[1])

                for (var i = 0; i < navigationModel.count; i++) {
                    var point = navigationModel.get(i);
                    screenPoint = getMappointFromCoord(point.lat, point.lon)
                    ctx.lineTo(screenPoint[0], screenPoint[1])

                }
                ctx.lineWidth = 2;
                ctx.strokeStyle = "#fffce94f";
                ctx.stroke();

            }


            console.timeEnd("canvas-onPaint")

        }
    }



    Waypoint {
        id: planePosition

        property bool positionValid: false;
        property real speed: 0; // speed in m/s
        mapx: map.x;
        mapy: map.y;
        centerY: planePosition.height - planePosition.width/2

        waypointType: positionValid ? "position-indicator" :  "position-indicator-red";

        onLatitudeChanged: {
            console.time("latitude changed")
            // remove point from list if we are flying trough it;
            if (navigationModel.count > 0) {
                var item = navigationModel.get(0);
                var distance = F.getDistanceTo(item.lat, item.lon, latitude, longitude);
                if (distance < pointRemoveDistance) {
                    navigationModel.remove(0, 1);
                }
            }

            // compute future flight position
            var futurePoint = F.getCoordByDistanceBearing(latitude, longitude, azimuth, speed*timeForFuturePos);
            var geohere = [latitude, longitude]
            var geonext = [futurePoint.lat, futurePoint.lon]

            if (air !== undefined && air.airspace !== undefined) {
                var airspaces = air.airspace;
                warningModel.clear();

                for (var i = 0; i < airspaces.length; i++) {
                    var airspace = airspaces[i];
                    var geometry = airspace.geometry;

                    var found = false;
                    var distance = 0;

                    // skip according to bounding box;

                    if (!((geonext[0] < geometry.min_lat) || (geonext[1] < geometry.min_lon) || (geonext[0] > geometry.max_lat) || (geonext[1] > geometry.max_lon))) {

                        if (F.pointInPolygon(geometry.points, geonext)) {
                            found = true;
                            distance = timeForFuturePos;
                        }
                    }

                    if (!((geohere[0] < geometry.min_lat) || (geohere[1] < geometry.min_lon) || (geohere[0] > geometry.max_lat) || (geohere[1] > geometry.max_lon))) {

                        if (F.pointInPolygon(geometry.points, geohere)) {
                            found = true;
                            distance = 0;
                        }

                    }

                    if (found) {
                        var nicename = (airspace.designator !== airspace.name) ? (airspace.designator + " " + airspace.name) : airspace.designator ;
                        warningModel.append({
                                                "type": "POLY",
                                                "subtype": airspace.type,
                                                "name": nicename,
                                                "designator": airspace.designator,
                                                "lat": geohere[0],
                                                "lon": geohere[1],
                                                "low": geometry.lowv + " " + geometry.lowu + " " + geometry.lowr,
                                                "up": geometry.upv + " " + geometry.upu + " " + geometry.upr,
                                                "ele": "0",
                                                "eleu": "FT",
                                                "directions": "",
                                                "events": airspace.tempdelta.length,
                                                "distance" : distance,

                                            })
                    }
                }
            }


            console.timeEnd("latitude changed")

            // repaint trajectory;
            trajectoryCanvasRequestUpdate();
        }
    }


    Rectangle {
        id: scaleBar
        anchors.right: parent.right
        anchors.rightMargin: 16
        anchors.topMargin: 16
        anchors.top: parent.top
        color: "black"
        border.width: 1
        border.color: "white"
        smooth: false
        height: 4
        width: scaleBarLength[0]
    }

    Text {
        text: F.formatDistance(scaleBarLength[1], {'distanceUnit':'m'})
        anchors.horizontalCenter: scaleBar.horizontalCenter
        anchors.top: scaleBar.bottom
        anchors.topMargin: 8
        style: Text.Outline
        styleColor: "white"
        font.pixelSize: 24
    }

    Repeater {
        model: airportModel;
        delegate: Waypoint {
            targetPoint: getMappointFromCoord(model.lat, model.lon)
            waypointType: "Airport"
            mapx: map.x;
            mapy: map.y;
            azimuth: (model.mainDirection*10)
        }
    }


    PinchArea {
        id: pincharea;

        property double __oldZoom;
        property double __oldAngle;
        //        property variant __lastTimestamp;

        anchors.fill: parent;

        function calcZoomDelta(p) {
            var newZoomLevel = (Math.log(p.scale)/Math.log(2)) + __oldZoom;
            pinchmap.setZoomLevelPoint(newZoomLevel, p.center.x, p.center.y);
            if (rotationEnabled) {
                rot.angle = __oldAngle + p.rotation
            }
            pan(p.previousCenter.x - p.center.x, p.previousCenter.y - p.center.y);
        }

        onPinchStarted: {
            __oldZoom = pinchmap.zoomLevel;
            __oldAngle = rot.angle
        }

        onPinchUpdated: {
            /// maximally 5 updates per second
            //            var now = new Date().getTime();
            //            var timeDiff = Math.abs(now - pincharea.__lastTimestamp);
            //            if (timeDiff < 200) {
            //                return;
            //            }
            //            pincharea.__lastTimestamp = now;


            calcZoomDelta(pinch);
        }

        onPinchFinished: {
            calcZoomDelta(pinch);
            panEnd();
            canvasRequestUpdate()
        }


        MouseArea {
            id: mousearea;

            property bool __isPanning: false;
            property int __lastX: -1;
            property int __lastY: -1;
            property int __firstX: -1;
            property int __firstY: -1;
            property bool __wasClick: false;
            property int maxClickDistance: 100;
            property real appendClickDistance: 30;

            anchors.fill : parent;

            onWheel: {
                if (wheel.angleDelta.y > 0) {
                    setZoomLevelPoint(pinchmap.zoomLevel + 0.25, wheel.x, wheel.y);
                } else {
                    setZoomLevelPoint(pinchmap.zoomLevel - 0.25, wheel.x, wheel.y);
                }
                canvasRequestUpdate()
            }


            onPressed: {
                pannedManually()
                __isPanning = true;
                __lastX = mouse.x;
                __lastY = mouse.y;
                __firstX = mouse.x;
                __firstY = mouse.y;
                __wasClick = true;
            }

            onReleased: {

                __isPanning = false;
                if ( __wasClick) {

                    clickedModel.clear();
                    var pos = getCoordFromScreenpoint(mouse.x, mouse.y);
                    clickedModel.append({
                                            "type": "POI",
                                            "subtype": "",
                                            //% "Selected place"
                                            "name": "Zvolené místo",
                                            "designator": "",
                                            "lat": pos[0],
                                            "lon": pos[1],
                                            "ele": "0",
                                            "eleu": "FT",
                                            "directions": "",
                                            "up": "",
                                            "low": "",
                                            "events": 0,
                                        })

                    // find airports/navigation points near to position of click

                    for (var i = 0; i < airportModel.count; i++) {
                        var item = airportModel.get(i);
                        var screen = getScreenpointFromCoord(item.lat, item.lon)
                        var distance = F.euclidDistance(screen[0], screen[1], mouse.x, mouse.y);
                        if (distance < appendClickDistance) {
                            clickedModel.append({
                                                    "type": item.type,
                                                    "subtype": "",
                                                    "name": item.designator,
                                                    "designator": item.designator,
                                                    "lat": item.lat,
                                                    "lon": item.lon,
                                                    "ele": item.ele,
                                                    "eleu": item.eleu,
                                                    "directions": item.directions,
                                                    "up": "",
                                                    "low": "",
                                                    "events": item.tempdelta.length,
                                                })
                        }
                    }

                    var geo = getCoordFromScreenpoint(mouse.x, mouse.y);
                    clickedIntoMap(geo[0], geo[1])

                    // search trough

                    if ((air !== undefined) && (air.airspace !== undefined)) {
                        var airspaces = air.airspace;
                        for (var i = 0; i < airspaces.length; i++) {
                            var airspace = airspaces[i];
                            var geometry = airspace.geometry;

                            // skip according to bounding box;
                            if ((geo[0] < geometry.min_lat) || (geo[1] < geometry.min_lon) || (geo[0] > geometry.max_lat) || (geo[1] > geometry.max_lon)) {
                                continue;
                            }

                            var nicename = (airspace.designator !== airspace.name) ? (airspace.designator + " " + airspace.name) : airspace.designator ;
                            if (F.pointInPolygon(geometry.points, geo)) {
                                clickedModel.append({
                                                        "type": "POLY",
                                                        "subtype": airspace.type,
                                                        "name": nicename,
                                                        "designator": airspace.designator,
                                                        "lat": geo[0],
                                                        "lon": geo[1],
                                                        "low": geometry.lowv + " " + geometry.lowu + " " + geometry.lowr,
                                                        "up": geometry.upv + " " + geometry.upu + " " + geometry.upr,
                                                        "ele": "0",
                                                        "eleu": "FT",
                                                        "directions": "",
                                                        "events": airspace.tempdelta.length,

                                                    })
                            }
                        }
                    }

                } else {
                    panEnd();
                }
                canvasRequestUpdate();
            }

            onPositionChanged: {
                if (__isPanning) {
                    var dx = mouse.x - __lastX;
                    var dy = mouse.y - __lastY;
                    pan(-dx, -dy);
                    __lastX = mouse.x;
                    __lastY = mouse.y;
                    /*
                    once the pan threshold is reached, additional checking is unnecessary
                    for the press duration as nothing sets __wasClick back to true
                    */
                    if (__wasClick && Math.pow(mouse.x - __firstX, 2) + Math.pow(mouse.y - __firstY, 2) > maxClickDistance) {
                        __wasClick = false;
                    }
                }
            }

            onCanceled: {
                __isPanning = false;
            }
        }

    }




    Item {
        anchors.fill: parent;
        focus: true
        Component.onCompleted: focus = true; // the previous line is not working

        Keys.onPressed: {
            //          console.log ("KEY pressed")
        }
    }



    FileReader {
        id: filereader;
    }


    //    Timer {
    //        id: canvasLateUpdate;
    //        repeat: true;
    //        running: true;
    //        interval: 500;

    //        property int requests: 0;
    //        property bool changedInLastCycle: false;
    //        onRequestsChanged: {
    //            if (requests !== 0) {
    //                changedInLastCycle = true;
    //            }
    //        }

    //        onTriggered:  {
    //            if ((requests > 0) && changedInLastCycle) {
    //                requests = 0;
    //                canvas.visible = true;
    //                canvas.requestPaint();
    //            }
    //            changedInLastCycle = false;
    //        }
    //    }

    function trajectoryCanvasRequestUpdate() {
        //        canvasLateUpdate.requests++;
        canvasRequestUpdate();
    }

    function canvasRequestUpdate() {
        //        canvas.visible = false;
        //        canvasLateUpdate.requests++;
        canvas.requestPaint();
    }

}
