import QtQuick 2.3
import "Theme.js" as Theme

Rectangle {

    color: Theme.background_color;
    visible: false;
    property var airfieldsModel;
    property var navModel;

    signal showNav();

    onVisibleChanged: {
        if (visible) {
            filterModel(searchField.text)
            searchField.forceActiveFocus();
            Qt.inputMethod.show();
        } else {
            Qt.inputMethod.hide();
        }
    }

    Text {
        id: pageHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: Theme.margin
        horizontalAlignment: Text.AlignRight;
        //% "Search"
        text: "Vyhledávání"
        color: Theme.header_color
        font.pixelSize: Theme.header_pixelSize
    }


    DialogInput {
        id: searchField
        anchors.top: pageHeader.bottom
        placeholderText: "Zadejte hledaného letiště";
        onTextChanged: {
            filterModel(text)
        }

    }

    ListModel {
        id: filteredModel;
    }

    function filterModel(search_text) {
        if (airfieldsModel === undefined) {
            return;
        }

        filteredModel.clear();

        var reg_exp = new RegExp(search_text, "i");

        for (var i = 0; i < airfieldsModel.count; i++) {
            var item = airfieldsModel.get(i);
            var match = reg_exp.exec(item.designator);
            if (match !== null) {
                filteredModel.append(item);
            }
        }
    }

    ListView {
        id: searchListView
        anchors.top: searchField.bottom
        anchors.left: parent.left;
        anchors.right: parent.right;
        anchors.bottom: parent.bottom;
        anchors.topMargin: Theme.margin
        anchors.bottomMargin: Theme.margin
        clip: true;
        spacing: Theme.marginSmall

        model: filteredModel
        delegate: DialogButton {
            anchors.left: parent.left;
            anchors.right: parent.right;
            title: model.designator
            subtitle: model.ele + " " + model.eleu + " - " + model.directions
            icon: "./data/AD.png"
            onClicked: {
                navModel.append({
                                    "name": model.designator,
                                    "type": model.type,
                                    "lat": parseFloat(model.lat, 10),
                                    "lon": parseFloat(model.lon, 10),
                                })

                showNav();
                searchField.text = "";
            }

        }
    }

}
