import QtQuick 2.0
import QtQuick.Controls 1.2
import "Theme.js" as Theme
import "functions.js" as F

Rectangle {

    color: Theme.background_color;
    visible: false;
    property alias rotationEnabled: rotationCheckbox.checked;
    property alias timestampBeginPosition: beginPosition.timestamp
    property alias timestampEndPosition: endPosition.timestamp

    signal showMapsPage();
    signal reloadNotams();


    Text {
        id: pageHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: Theme.margin
        horizontalAlignment: Text.AlignRight;
        text: "Nastavení"
        color: Theme.header_color
        font.pixelSize: Theme.header_pixelSize
    }


    Flickable {
        anchors.top: pageHeader.bottom
        anchors.left: parent.left;
        anchors.right: parent.right;
        anchors.bottom: parent.bottom;
        contentHeight: column.height
        anchors.topMargin: Theme.margin
        anchors.bottomMargin: Theme.margin
        clip: true;


        Column {
            id: column
            anchors.left: parent.left
            anchors.right: parent.right
            spacing: Theme.marginSmall

            function dnotam_reload_accepted() {
                beginPosition.timestamp = F.dateStringToTimestamp(beginPosition.text);
                endPosition.timestamp = F.dateStringToTimestamp(endPosition.text);
                reloadNotams();
            }


            DialogButton {
                title: "NOTAMy"
                subtitle: "Vynutí stažení aktualních dat"
                onClicked: column.dnotam_reload_accepted();
            }

            DialogInput {
                id: beginPosition
                placeholderText: "2014-01-01 00:00:00"
                property int timestamp: (new Date().getTime()/86400000).toFixed(0)*86400;
                text: F.timestampToDateString(timestamp)
                validator: RegExpValidator {
                    regExp: /(\d?\d?\d?\d?)-?(\d?\d?)-?(\d?\d?) ?(\d?\d?):?(\d?\d?):?(\d?\d?)/;
                }
                onAccepted: column.dnotam_reload_accepted();
            }

            DialogInput {
                id: endPosition
                placeholderText: "2015-01-01 00:00:00"
                property int timestamp: (new Date().getTime()/86400000).toFixed(0)*86400+86400;
                text: F.timestampToDateString(timestamp)
                validator: RegExpValidator {
                    regExp: /(\d?\d?\d?\d?)-?(\d?\d?)-?(\d?\d?) ?(\d?\d?):?(\d?\d?):?(\d?\d?)/;
                }
                onAccepted: column.dnotam_reload_accepted();

            }



            DialogButton {
                title: "Mapy"
                subtitle: "Zobrazí seznam dostupných regionů a stav stahování"
                onClicked:  {
                    showMapsPage();
                }
            }

            DialogCheckbox {
                id: rotationCheckbox
                title: "Otáčení"
                subtitle: "Zamkne automatické otáčení ve směru letu";
            }


        }


    }


}
