import QtQuick 2.3
import cz.mlich 1.0
import "Theme.js" as Theme
import "functions.js" as F

Rectangle {
    id: settingsMapsPage
    color: Theme.background_color;
    visible: false;

    property alias regionsModel: regions;
    signal backClicked();

    ListModel {
        id: regions;
    }


    Text {
        id: pageHeader
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: Theme.margin
        horizontalAlignment: Text.AlignRight;
        text: "Mapy"
        color: Theme.header_color
        font.pixelSize: Theme.header_pixelSize
    }


    ListView {
        anchors.top: pageHeader.bottom
        anchors.left: parent.left;
        anchors.right: parent.right;
        anchors.bottom: backButton.top;
        anchors.topMargin: Theme.margin
        anchors.bottomMargin: Theme.margin
        clip: true;
        spacing: Theme.marginSmall

        model: regions
        delegate: DialogButton {
            anchors.left: parent.left;
            anchors.right: parent.right;

            title: model.name
            subtitle:
                F.formatDate(new Date(model.lastUpdate*1000)) + " - " +
                F.formatSize(model.estimated_size) + " - " +
                getDownloadStatusString(model.download_status) +
                ((model.download_status === "inProgress") ? ": " + ((1-(downloader.count / model.estimated_count))*100).toFixed(1) + "%" : "")
            onClicked: {
//                console.log(JSON.stringify(regions.get(index)))
                switch (model.download_status) {
                case "inProgress":
                    downloader.cancel();
                    regions.setProperty(index, "download_status", "notDownloaded")
                    break
                case "queued":
                    regions.setProperty(index, "download_status", "notDownloaded")
                    break;
                case "unknown":
                case "error":
                case "notDownloaded":
                    regions.setProperty(index, "download_status", "queued")
                    break;
                }

            }
        }
    }

    function getDownloadStatusString(s) {
        switch(s) {
        case "done":
            return "Hotovo";
        case "queued":
            return "Ve frontě";
        case "inProgress":
            return "Probíhá stahování";
        case "notDownloaded":
            return "Není staženo";
        case "error":
            return "Chyba stahování";
        case "unknown":
        default:
            return "Neznámo";
        }
    }

    property int zoom_min: 3;
    property int zoom_max: 13;

    function createList(lat_min, lon_min, lat_max, lon_max) {
        var i = 0;
        for (var z = zoom_min; z < zoom_max; z++) {
            var maxVals = F.deg2num2(lat_max, lon_max, z)
            var minVals = F.deg2num2(lat_min, lon_min, z)

            var x1 = parseInt(Math.floor(minVals[0]));
            var x2 = parseInt(Math.ceil(maxVals[0]));
            var y2 = parseInt(Math.ceil(minVals[1]));
            var y1 = parseInt(Math.floor(maxVals[1]));

            for (var x = x1; x < x2; x++) {
                for (var y = y1; y < y2; y++) {
                    downloader.append(F.getMapTile(MAP_REMOTE_URL, x, y, z), F.getMapTile(MAP_LOCAL_STORAGE, x, y, z));
                    i++;
                }
            }
        }
        return i;
    }



    DialogButton {
        id: backButton
        anchors.bottom: parent.bottom;
        // FIXME pouzit hardwarove zpet a gesto
        title: "Zpět"
        onClicked: backClicked();
    }

    Timer {
        running: settingsMapsPage.visible;
        repeat: true;
        interval: 1000;
        onTriggered: {
            var download_in_progress = false;
            for (var i = 0; i < regions.count; i++) {
                var item = regions.get(i);
                if (item.download_status  === "inProgress") {
                    download_in_progress = true;
                }
            }
            if (!download_in_progress) {
                for (var i = 0; i < regions.count; i++) {
                    var item = regions.get(i);
                    if (item.download_status  === "queued") {
                        regions.setProperty(i, "download_status", "inProgress");
                        downloader.modelIndex = i;
                        var tileCount = createList(item.lat_min, item.lon_min, item.lat_max, item.lon_max);
                        regions.setProperty(i, "estimated_count", tileCount);


                        break;
                    }

                }
            }

        }
    }


    Downloader {
        id: downloader;
        property int modelIndex: -1;
        onCountChanged: {
            if ((modelIndex >= 0) && (modelIndex < regions.count) && (downloader.count <= 0)) { // valid index
                var old = regions.get(modelIndex)
                if (old.download_status === "inProgress") {
                    regions.setProperty(modelIndex, "download_status", "done");
                }
            }
        }

        onErrorStringChanged: {
            if ((modelIndex >= 0) && (modelIndex < regions.count)) { // valid index
                if (errorString !== "") {
                    regions.setProperty(modelIndex, "download_status", "error");
                }
                regions.setProperty(modelIndex, "download_error", errorString);
            }
        }
    }





}
